use crate::config::config::STATIC_WS_PWD;
use crate::handles::close_connection::handle_connection_error;
use crate::handles::handle_messages::handle_other_message;
use crate::handles::handshake::handle_handshake;
use crate::handles::heartbeat::{handle_heartbeat, heart_resp};
use crate::handles::redis::{insert_this_connection, remove_this_connection};
use crate::typing::used_typed::{Connection, ConnectionMap, TaskMap};
use crate::utils::json_utils::parse_message;
use futures::{SinkExt, StreamExt};
use lazy_static::lazy_static;
use std::collections::HashMap;
use std::sync::{Arc, RwLock};
use std::time::{Duration, Instant};
use tokio::sync::RwLock as AsyncRwLock;
use tokio::time;
use tokio_tungstenite::accept_hdr_async;
use tungstenite::handshake::server::{Request, Response};
use tungstenite::{Error, Message};
use crate::handles::online_users_update::send_online_users_resp;

lazy_static! {
    pub static ref CONNECTIONS: ConnectionMap = Arc::new(AsyncRwLock::new(HashMap::new()));
    static ref TASKS: TaskMap = Arc::new(RwLock::new(HashMap::new()));
}
lazy_static! {
    pub static ref ONLINE_USERS: Arc<AsyncRwLock<HashMap<String, String>>> = Arc::new(AsyncRwLock::new(HashMap::new()));
}

// 关闭之前绑定的 WebSocket 连接并取消对应的任务
async fn close_existing_connection(from_id: &str) {
    let task_to_abort = {
        let mut tasks = TASKS.write().unwrap();
        tasks.remove(from_id)
    };

    if let Some(task) = task_to_abort {
        task.abort();
    }

    let old_connection = {
        let mut connections = CONNECTIONS.write().await;
        let already_done = connections.get(&from_id.to_string());
        println!(
            "关闭之前绑定的 WebSocket 连接: {} {:?}",
            from_id, already_done
        );
        connections.remove(from_id)
    };

    if let Some(mut old_connection) = old_connection {
        // 尝试优雅地关闭旧连接
        if let Err(e) = old_connection.sender.close().await {
            println!("关闭旧的 WebSocket 发送器时出错: {}", e);
        }
        println!("关闭旧的 WebSocket 连接: {}", from_id);
    }

    // 从 Redis 中移除该用户的信息
    if let Err(e) = remove_this_connection(from_id).await {
        println!("从 Redis 中移除用户信息时出错: {}", e);
    }
}

pub(crate) async fn handle_client(stream: tokio::net::TcpStream) -> Result<(), Error> {
    let must_existed_params = ["deviceId", "fromId", "wsPwd"];
    let mut connection_params = None;

    let ws_stream = match accept_hdr_async(stream, |req: &Request, resp| {
        match handle_handshake(req, &must_existed_params, STATIC_WS_PWD) {
            Ok(params) => {
                connection_params = Some(params);
                Ok(resp)
            }
            Err(error_msg) => {
                println!("{}", error_msg);
                let error_resp = Response::builder()
                    .status(400)
                    .header("Content-Type", "text/plain")
                    .body(Some(error_msg))
                    .unwrap();
                Err(error_resp)
            }
        }
    })
        .await
    {
        Ok(ws) => ws,
        Err(e) => {
            println!("WebSocket握手失败: {}", e);
            return Ok(());
        }
    };

    let (sender, receiver) = ws_stream.split();

    if let Some(params) = connection_params {
        if let Some(from_id) = params.get("fromId") {
            let from_id = from_id.clone();
            let from_id_clone = from_id.clone(); // 新增：克隆一份 from_id 用于闭包

            // 检查 Redis 中是否已经存在该 fromId
            close_existing_connection(&from_id).await;

            // 将新连接添加到全局连接映射
            {
                let mut connections = CONNECTIONS.write().await;
                connections.insert(from_id.clone(), Connection { sender, receiver, from_id: from_id.to_string() });
            }

            // 将该用户的信息插入到 Redis 中
            if let Err(e) = insert_this_connection(&from_id, &params).await {
                println!("将用户信息插入到 Redis 中时出错: {}", e);
            }

            // 准备更新用户链接
            if let Err(e) = send_online_users_resp().await {
                println!("在处理新用户id {} 之时广播，处理在线用户列表出错了：{:?}", &from_id,e);
            } else {
                println!("广播消息 来源id {} 成功", &from_id);
            }

            let task = tokio::spawn(async move {
                let mut last_heartbeat_time = Instant::now();
                loop {
                    let current_connection = {
                        let mut connections = CONNECTIONS.write().await;
                        connections.remove(&from_id_clone) // 移除连接，将其从 map 中取出
                    };
                    if let Some(connection) = current_connection {
                        let (mut receiver_ref, mut sender_ref) =
                            (connection.receiver, connection.sender);
                        tokio::select! {
                            // 处理消息接收
                            maybe_msg = receiver_ref.next() => {
                                match maybe_msg {
                                    Some(Ok(msg)) => {
                                        if msg.is_text() {
                                            let text = msg.to_text().unwrap();
                                            match parse_message(text) {
                                                Ok(data) => {
                                                    match data.msg_type.as_str() {
                                                        "Heart" => {
                                                            println!("收到客户端心跳消息 {:?}", &data);
                                                            handle_heartbeat(&mut last_heartbeat_time);
                                                            if let Ok(json_str) = heart_resp(&from_id_clone) {
                                                                if let Err(e) = sender_ref.send(Message::text(json_str)).await {
                                                                    println!("发送心跳信息失败: {}", e);
                                                                    handle_connection_error(&from_id_clone, &CONNECTIONS, &TASKS).await;
                                                                    break;
                                                                }
                                                            }
                                                        },
                                                        "GetOnlineUserList" => {
                                                            println!("收到客户端获取在线用户列表 {:?}", &data);
                                                            if let Err(e) = send_online_users_resp().await {
                                                                println!("处理在线用户列表出错了：{:?}", e);
                                                                handle_connection_error(&from_id_clone, &CONNECTIONS, &TASKS).await;
                                                                break;
                                                            }
                                                        },
                                                        _ => {
                                                            if let Err(e) = handle_other_message(&mut sender_ref, &data, &from_id_clone).await {
                                                                println!("Failed to handle other message: {}", e);
                                                                handle_connection_error(&from_id_clone, &CONNECTIONS, &TASKS).await;
                                                                break;
                                                            }
                                                        }
                                                    }
                                                }
                                                Err(e) => {
                                                    println!("解析JSON数据出错: {}", e);
                                                }
                                            }
                                        }
                                    }
                                    Some(Err(e)) => {
                                        println!("接受客户端消息出错: {}", e);
                                        handle_connection_error(&from_id_clone, &CONNECTIONS, &TASKS).await;
                                        break;
                                    }
                                    None => {
                                        println!("客户端断开连接");
                                        handle_connection_error(&from_id_clone, &CONNECTIONS, &TASKS).await;
                                        break;
                                    }
                                }
                            }
                            // 处理心跳超时
                            _ = time::sleep_until(tokio::time::Instant::from(last_heartbeat_time + Duration::from_secs(20))) => {
                                println!("用户id-{} 20秒内没有发送心跳，挂断连接", from_id_clone); // 使用克隆后的 from_id
                                break;
                            }
                        }
                        // 处理完一轮后，若连接还未断开，将其重新放回 CONNECTIONS
                        {
                            let mut connections = CONNECTIONS.write().await;
                            connections.insert(
                                from_id_clone.clone(),
                                Connection {
                                    sender: sender_ref,
                                    receiver: receiver_ref,
                                    from_id: from_id_clone.to_string(),
                                },
                            );
                        }
                    } else {
                        break;
                    }
                }
                println!("断开与用户id: {}，连接", from_id_clone); // 使用克隆后的 from_id
                // 从全局连接映射中移除该连接
                {
                    let mut connections = CONNECTIONS.write().await;
                    connections.remove(&from_id_clone); // 使用克隆后的 from_id
                }
                // 从 Redis 中移除该用户的信息
                if let Err(e) = remove_this_connection(&from_id_clone).await {
                    println!("从 Redis 中移除用户信息时出错: {}", e);
                }
            });

            // 将任务句柄存储到全局任务映射中
            let mut tasks = TASKS.write().unwrap();
            tasks.insert(from_id, task); // 使用原始的 from_id
        }
    } else {
        println!("无法获取连接参数");
    }

    Ok(())
}